#include "wrdef.h"
#include "wrio.h"
#include "wrapi.h"

static const BYTE Modes[] = {0x5e, 0x5f, 0x60, 0x61, 0x5e, 0x5e, 0x61, 0x60};

BOOL SetMode1000(int hRadio, int iMode)
{
	LPRADIOSETTINGS rs = RadioSettings[hRadio];

	if (!WriteMcuByte(hRadio, Modes[iMode]))
		return FALSE;

	// if was or will be in a mode that does frequency mangling, update the frequency
	if ((iMode == RMD_CW) || (iMode == RMD_LSB) ||
		(iMode == RMD_USB) || (iMode == RMD_FMW) ||
		(rs->iCurMode == RMD_CW) || (rs->iCurMode == RMD_LSB) ||
		(rs->iCurMode == RMD_USB) || (rs->iCurMode == RMD_FMW))
	{
		int oMode = rs->iCurMode;
		rs->iCurMode = iMode;

		if (((oMode == RMD_CW) || (oMode == RMD_LSB) || (oMode == RMD_USB)) &&
			!((iMode == RMD_CW) || (iMode == RMD_LSB) || (iMode == RMD_USB)))
			rs->lpSetBfoProc(hRadio, 0);	// reset BFO if not in SSB anymore

		SetFrequency(hRadio, rs->dwFreq);
	}

	// set narrow filter if in SSB mode
	if (rs->riInfo.wHWVer >= RHV_1500)
		WriteMcuByte(hRadio, ((iMode == RMD_CW) || (iMode == RMD_LSB) ||
			(iMode == RMD_USB)) ? 0x6b : 0x6c);

	rs->iCurMode = iMode;

	return TRUE;
}

BOOL SetMode2000(int hRadio, int iMode)
{
	LPRADIOSETTINGS rs = RadioSettings[hRadio];

	if (!WriteMcuByte(hRadio, Modes[iMode]))
		return FALSE;

	if (!WriteMcuByte(hRadio, (iMode >= RMD_FMM) ? 0x59 : 0x58))
		return FALSE;

	// if was or will be in a mode that does frequency mangling, update the frequency
	if (rs->riInfo.wHWVer >= RHV_1500)
	{
		if ((iMode == RMD_CW) || (iMode == RMD_LSB) ||
			(iMode == RMD_USB) || (iMode == RMD_FMW) ||
			(rs->iCurMode == RMD_CW) || (rs->iCurMode == RMD_LSB) ||
			(rs->iCurMode == RMD_USB) || (rs->iCurMode == RMD_FMW))
		{
			int oMode = rs->iCurMode;
			rs->iCurMode = iMode;

			if (((oMode == RMD_CW) || (oMode == RMD_LSB) || (oMode == RMD_USB)) &&
				!((iMode == RMD_CW) || (iMode == RMD_LSB) || (iMode == RMD_USB)))
				rs->lpSetBfoProc(hRadio, 0);	// reset BFO if not in SSB anymore

			SetFrequency(hRadio, rs->dwFreq);
		}

		if (!(rs->riInfo.dwFeatures & RIF_AGC))
			rs->lpSetAgcProc(hRadio, iMode != RMD_FMN);

		// set narrow filter if in SSB mode
		WriteMcuByte(hRadio, ((iMode == RMD_CW) || (iMode == RMD_LSB) ||
			(iMode == RMD_USB)) ? 0x70 : 0x71);
	}
	rs->iCurMode = iMode;

	return TRUE;
}
